const { modulo11, calculateVerifyingDigitMod10 } = require('./utils');
const PaymentSlip = require('./PaymentSlip');

class BankBill extends PaymentSlip {

  buildBarcode(line) {
    let b = line.splice(0, 4).join('') + line.splice(15, 25).join('') + line.splice(0, 15).join('');
    b = [...b];
    const area1 = b.splice(0, 9);
    const digit1 = calculateVerifyingDigitMod10([...area1]);
    const area2 = b.splice(0, 10);
    const digit2 = calculateVerifyingDigitMod10([...area2]);
    const area3 = b.splice(0, 10);
    const digit3 = calculateVerifyingDigitMod10([...area3]);
    const barcode = area1.join('') + digit1.toString() + area2.join('') + digit2.toString() + area3.join('') + digit3.toString() + b.join('');
    return barcode;
  }

  removeVerifyingDigits(line) {
    line.splice(9, 1);
    line.splice(19, 1);
    line.splice(29, 1);
    return line;
  }

  generateBarcodeFromLine(line) {
    const barcode = [...line].splice(0, 4).join('') + [...line].splice(29, 15).join('') + [...line].splice(4, 25).join('');
    return barcode;
  }

  validateBarcode() {
    const code = this.removeVerifyingDigits([...this.barcode]);

    if (code.join('').replace(/\D/g, '').length !== 44) {
      return false;
    }

    let barcode = this.generateBarcodeFromLine(code)

    barcode = barcode.split('');

    const verifyingDigit = barcode.splice(4, 1);
    const valid = modulo11(barcode, verifyingDigit);

    return valid;
  }

  calculateDueDate() {
    const barcode = [...this.barcode];

    const DATE_POSITION = 33;
    const DATE_SIZE = 4;

    const dateArray = barcode.splice(DATE_POSITION, DATE_SIZE);
    const dateString = dateArray.join('');

    if (dateString === '0000') {
      return null;
    }

    const baseDate = new Date('1997-10-07 GMT-0300'); // BACEN base date to calculate due dates.
    baseDate.setDate(baseDate.getDate() + parseInt(dateString, 10));
    return baseDate;
  }

  getAmount() {
    const barcode = [...this.barcode];
    const AMOUNT_POSITION = 37;
    const AMOUNT_SIZE = 10;

    const amountArray = barcode.splice(AMOUNT_POSITION, AMOUNT_SIZE);

    const intAmount = parseInt(amountArray.join(''), 10);
    const amount = intAmount / 100;

    return amount;
  }
}

module.exports = BankBill;
