const {
  modulo10,
  modulo11,
  calculateVerifyingDigitMod11,
  calculateVerifyingDigitMod10
} = require('./utils');
const PaymentSlip = require('./PaymentSlip');


class TaxBill extends PaymentSlip {

  selectModule(barcode, building=false) {
    let module;
    if (barcode[2] === '6' || barcode[2] === '7') {
      if (building) {
        module = calculateVerifyingDigitMod10;
      } else {
        module = modulo10;
      }
    } else if (barcode[2] === '8' || barcode[2] === '9') {
      if (building) {
        module = calculateVerifyingDigitMod11;
      } else {
        module = modulo11;
      }
    } else {
      module = false;
    }

    return module;
  }

  buildBarcode(line) {
    const building = true;
    const module = this.selectModule(line, building);
    let barcode = '';

    while (barcode.length < 48) {
      const part = line.splice(0, 11).join('');
      const digit = module([...part]).toString();
      barcode +=  part + digit;
    }

    return barcode;
  }

  validateBarcode(barcode) {
    const module = this.selectModule(barcode);

    if (!module) return false;

    let result = true;

    while (barcode.length > 0) {
      const m = module(barcode.splice(0, 11), barcode.splice(0, 1));
      result = result && m;
    }
    return result;
  }

  removeVerifyingDigits(barcode) {
    const FIRST_VERIFYING_DIGIT_POSITION = 11;
    const SECOND_VERIFYING_DIGIT_POSITION = 22;
    const THIRD_VERIFYING_DIGIT_POSITION = 33;
    const FOURTH_VERIFYING_DIGIT_POSITION = 44;

    // remove verifying digits
    barcode.splice(FIRST_VERIFYING_DIGIT_POSITION, 1);
    barcode.splice(SECOND_VERIFYING_DIGIT_POSITION, 1);
    barcode.splice(THIRD_VERIFYING_DIGIT_POSITION, 1);
    barcode.splice(FOURTH_VERIFYING_DIGIT_POSITION, 1);

    return barcode;
  }

  getAmount() {
    const barcode = this.removeVerifyingDigits([...this.barcode]);

    const VALUE_POSITION = 4;
    const VALUE_LENGTH = 11;
    const str_value = [...barcode].splice(VALUE_POSITION, VALUE_LENGTH).join('');

    let amount = parseFloat(str_value);
    amount /= 100;

    return amount;
  }

  formatDate(barcode) {
    const DATE_POSITION = 19;
    const DATE_LENGTH = 6;
    const dateArray = barcode.splice(DATE_POSITION, DATE_LENGTH);
    const dateString = dateArray.join('');
    const formattedDateString = `20${dateString[4]}${dateString[5]}-${dateString[2]}${dateString[3]}-${dateString[0]}${dateString[1]}`;
    const date = new Date(`${formattedDateString} GMT-0300`);
    return date;
  }

  formatDate2(barcode) {
    const DATE_POSITION = 19;
    const DATE_LENGTH = 8;
    const dateArray = barcode.splice(DATE_POSITION, DATE_LENGTH);
    const year = [...dateArray].splice(0, 4).join('');
    const month = [...dateArray].splice(4, 2).join('');
    const day = [...dateArray].splice(6, 2).join('');
    const formattedDateString = `${year}-${month}-${day}`;
    const date = new Date(`${formattedDateString} GMT-0300`);
    return date;
  }

  calculateDueDate() {
    const barcode = this.removeVerifyingDigits([...this.barcode]);

    let dueDate = this.formatDate([...barcode]);

    if (isNaN(dueDate))
      dueDate = this.formatDate2([...barcode]);

    if (isNaN(dueDate))
      dueDate = '';

    return dueDate;
  }
}

module.exports = TaxBill;
